@@ -283,7 +283,12 @@ impl ApiTypeDelegate {
 
 impl ApiTypeChild for ApiTypeDelegate {
     fn safe_ident(&self) -> String {
-        self.dart_api_type().to_case(Case::Snake)
+        match self {
+            ApiTypeDelegate::String => "String".to_string(),
+            ApiTypeDelegate::ZeroCopyBufferVecU8 => {
+                "ZeroCopyBuffer_".to_owned() + &self.get_delegate().dart_api_type()
+            }
+        }
     }
 
     fn dart_api_type(&self) -> String {
