@@ -1,21 +1,25 @@
+use convert_case::{Case, Casing};
+use log::debug;
+
 use crate::api_types::ApiType::*;
 use crate::api_types::*;
 use crate::generator_common::*;
-use convert_case::{Case, Casing};
 
 pub fn generate(
     api_file: &ApiFile,
     dart_api_class_name: &str,
+    dart_api_impl_class_name: &str,
     dart_wire_class_name: &str,
-    dart_wire_file_name: &str,
-) -> String {
-    let dart_functions = api_file
+) -> (String, String, String) {
+    let distinct_types = api_file.distinct_types();
+    debug!("distinct_types={:?}", distinct_types);
+
+    let dart_func_signatures_and_implementations = api_file
         .funcs
         .iter()
         .map(generate_api_func)
         .collect::<Vec<_>>();
-    let dart_structs = api_file
-        .distinct_types()
+    let dart_structs = distinct_types
         .iter()
         .filter_map(|ty| {
             if let StructRef(s) = ty {
@@ -26,37 +30,62 @@ pub fn generate(
         })
         .map(generate_api_struct)
         .collect::<Vec<_>>();
-    let dart_api2wire_funcs = api_file
-        .distinct_types()
+    let dart_api2wire_funcs = distinct_types
         .iter()
         .map(|ty| generate_api2wire_func(ty))
         .collect::<Vec<_>>();
-    let dart_api_fill_to_wire_funcs = api_file
-        .distinct_types()
+    let dart_api_fill_to_wire_funcs = distinct_types
         .iter()
         .map(|ty| generate_api_fill_to_wire_func(ty, api_file))
         .collect::<Vec<_>>();
-    let dart_wire2api_funcs = api_file
-        .distinct_types()
+    let dart_wire2api_funcs = distinct_types
         .iter()
         .map(|ty| generate_wire2api_func(ty, api_file))
         .collect::<Vec<_>>();
 
-    format!(
+    let header = format!(
         "{}
 
-        // ignore_for_file: non_constant_identifier_names, unused_element
+        // ignore_for_file: non_constant_identifier_names, unused_element, duplicate_ignore
 
         import 'dart:convert';
-        import 'dart:ffi' as ffi;
         import 'dart:typed_data';
 
-        import 'package:flutter_rust_bridge/flutter_rust_bridge.dart';
+        import 'package:flutter_rust_bridge/flutter_rust_bridge.dart';",
+        CODE_HEADER,
+    );
 
-        import '{}';
+    let api_class = format!(
+        "abstract class {} extends FlutterRustBridgeBase<{}> {{
+            factory {}(ffi.DynamicLibrary dylib) => {}.raw({}(dylib));
 
-        class {} extends DartRustBridgeBase<{}> {{
-            {}({} inner) : super(inner);
+            {}.raw({} inner) : super(inner);
+
+            {}
+        }}
+
+        {}
+
+        // ------------------------- Implementation Details -------------------------
+        ",
+        dart_api_class_name,
+        dart_wire_class_name,
+        dart_api_class_name,
+        dart_api_impl_class_name,
+        dart_wire_class_name,
+        dart_api_class_name,
+        dart_wire_class_name,
+        dart_func_signatures_and_implementations
+            .iter()
+            .map(|(sig, _)| sig.clone())
+            .collect::<Vec<_>>()
+            .join("\n\n"),
+        dart_structs.join("\n\n"),
+    );
+
+    let others = format!(
+        "class {} extends {} {{
+            {}.raw({} inner) : super.raw(inner);
 
             {}
 
@@ -67,27 +96,27 @@ pub fn generate(
             {}
         }}
 
-        // Section: dart structs
-        {}
-
         // Section: wire2api
         {}
         ",
-        CODE_HEADER,
-        dart_wire_file_name,
+        dart_api_impl_class_name,
         dart_api_class_name,
+        dart_api_impl_class_name,
         dart_wire_class_name,
-        dart_api_class_name,
-        dart_wire_class_name,
-        dart_functions.join("\n\n"),
+        dart_func_signatures_and_implementations
+            .iter()
+            .map(|(_, imp)| imp.clone())
+            .collect::<Vec<_>>()
+            .join("\n\n"),
         dart_api2wire_funcs.join("\n\n"),
         dart_api_fill_to_wire_funcs.join("\n\n"),
-        dart_structs.join("\n\n"),
         dart_wire2api_funcs.join("\n\n"),
-    )
+    );
+
+    (header, api_class, others)
 }
 
-fn generate_api_func(func: &ApiFunc) -> String {
+fn generate_api_func(func: &ApiFunc) -> (String, String) {
     let func_param_list = func
         .inputs
         .iter()
@@ -112,11 +141,8 @@ fn generate_api_func(func: &ApiFunc) -> String {
         })
         .collect::<Vec<_>>();
 
-    format!(
-        "Future<{}> {}({}) async {{
-            return execute((port) => inner.{}(port, {}), _wire2api_{});
-        }}
-        ",
+    let partial = format!(
+        "Future<{}> {}({})",
         func.output.dart_api_type(),
         func.name.to_case(Case::Camel),
         if func_param_list.is_empty() {
@@ -124,10 +150,19 @@ fn generate_api_func(func: &ApiFunc) -> String {
         } else {
             format!("{{{}}}", func_param_list.join(","))
         },
+    );
+
+    let signature = format!("{};", partial);
+
+    let implementation = format!(
+        "{} => execute((port) => inner.{}(port, {}), _wire2api_{});",
+        partial,
         func.wire_func_name(),
         wire_param_list.join(", "),
         func.output.safe_ident(),
-    )
+    );
+
+    (signature, implementation)
 }
 
 fn generate_api2wire_func(ty: &ApiType) -> String {
@@ -138,7 +173,7 @@ fn generate_api2wire_func(ty: &ApiType) -> String {
                 "return _api2wire_uint_8_list(utf8.encoder.convert(raw));".to_string()
             }
             ApiTypeDelegate::ZeroCopyBufferVecU8 => {
-                "return _api2wire_uint_8_list(raw.0);".to_string()
+                "return _api2wire_uint_8_list(raw);".to_string()
             }
         },
         PrimitiveList(_) => {
@@ -213,7 +248,7 @@ fn generate_api_fill_to_wire_func(ty: &ApiType, api_file: &ApiFile) -> String {
             .join("\n"),
         // skip
         Primitive(_) | Delegate(_) | PrimitiveList(_) | GeneralList(_) | Boxed(_) => {
-            return "".to_string()
+            return "".to_string();
         }
     };
 
